package NetworkCrawlerParser;

# This package provides routines for writing ioc network interface data
# to a MySQL database by parsing a nightly updated network data file. 

use DBI qw(:sql_types);
require Exporter;

use Mail::Mailer;
use File::stat;
use Cwd;

use LogUtil;

our @ISA = qw(Exporter);
our @EXPORT = qw( &email_log &send_email &get_modified_network_data 
    &get_all_network_data);
our @EXPORT_OK = qw();
our $VERSION = 1.00;
our %hostHash = ();
our $bootparams="/usr/local/iocapps/iocinfo/bootparams";

$email_body="";


#=============================================================================
# Public
sub get_modified_network_data {
  my $infileDiffSorted = $_[0];
  my $network_hits_ref = $_[1];

  my $ioc_id;
  my $ioc_row;
  my $select_line;
  my $line;
  my $iocEtypeHold = '';

  if ( ! open(INFILE,$infileDiffSorted) ) {
    email_log('error',"Cannot open sorted diff file $infileDiffSorted: $@");   
    return -1;
  }

  my $updateCount = 0;
  while($line = <INFILE>)
  {
    email_log('verbose',"Input line: $line");   
    $line =~ s/\015?\012//; # remove CR LF  characters

    ($updateType,$switch,$bladePort,$iocEtype,$connectStatus) = split(/\s+/,$line);

    # Use only network data for iocs
    if ( !defined $switch || $switch =~ m/^Switch/ ||
         !defined $connectStatus || !defined $iocEtype ||
         $iocEtype eq "connected" || $iocEtype eq "notconnect") {
      next;
    }

    # Get ioc name from input file
    # Cant use split because ioc may have embedded dashes
    next if substr($iocEtype,length($iocEtype)-2,1) ne "-";
    $ioc = substr($iocEtype,0,length($iocEtype)-2);
    $Etype = substr($iocEtype,length($iocEtype)-1,1);

    # Use only network data for iocs primary/secondary interfaces
    if ( $ioc =~ m/^hub/  || !defined $Etype || ( $Etype !~ m/s/ &&  $Etype !~ m/p/) )  {
      email_log('verbose',"$line");   
      next;
    }

    if ($iocEtypeHold ne '' && $iocEtype ne $iocEtypeHold ) { 
      push(@$network_hits_ref,[$iocHold,$EtypeHold,'','',0]);   
      $updateCount++;
      email_log('info',"Modified data: $iocEtypeHold OLD: $switchHold:$bladePortHold NEW: no data");   
    }

    # May have to add ioc prefix
    if ( $ioc !~ m/^ioc/ ) {
      email_log('warn',"Prefix `ioc` was added to $ioc");   
      $ioc =~ s/^/ioc/ 
    }


    if ($updateType eq "<") {
      $iocEtypeHold = $iocEtype;
      $iocHold = $ioc;
      $EtypeHold = $Etype;
      $switchHold = $switch;
      $bladePortHold = $bladePort;
    } else {
      ($blade,$port) = split(/\//,$bladePort);
      if ( $iocEtype eq $iocEtypeHold ) {
        if ( $switchHold ne $switch || $bladePortHold ne $bladePort ) {
          push(@$network_hits_ref,[$ioc,$Etype,$switch,$blade,$port]);   
          $updateCount++;
          email_log('info',"Modified data: $ioc-$Etype OLD: $switchHold:$bladePortHold NEW:$switch:$bladePort");   
        }
      } else {
        $updateCount++;
        push(@$network_hits_ref,[$ioc,$Etype,$switch,$blade,$port]);   
        email_log('info',"New data: $ioc-$Etype :$switch:$bladePort");   
      }
      $iocEtypeHold = '';
    }
  }
  close(INFILE);

  return $updateCount;
}

#=============================================================================
# Public
sub get_all_network_data {
  my $infile = $_[0];
  my $network_hits_ref = $_[1];
  my $ioc;
  my $host;

  email_log('info',"Replacing ALL ioc network data.");   

  if ( ! open(INFILE,$infile) ) {
    email_log('error',"Cannot open $infile: $@");   
    return -1;
  }

  createHostHash();

  my $line;
  while($line = <INFILE>)
  {
    email_log('verbose',"Input: $line");   
    $line =~ s/\015?\012//; # remove CR LF  characters

    ($switch,$bladePort,$iocEtype,$connectStatus) = split(/\s+/,$line);

    if ( !defined $switch || $switch =~ m/^Switch/ ||
         !defined $connectStatus || $bladePort =~ m/^---/  ||
         $iocEtype eq "connected" || $iocEtype eq "notconnect") {
      email_log('verbose',"Skipping: $line");   
      next;
    }

    ($blade,$port) = split(/\//,$bladePort);

    # Get ioc name from input file
    # Cant use split because ioc may have embedded dashes
    if (substr($iocEtype,length($iocEtype)-2,1) ne "-") {

      $host = $iocEtype;
      $host = 'ctlapps3l' if $host eq 'ctlapps3';
      my $softiocList=$hostHash{$host};
      if ( $softiocList ) {
        foreach $softioc (@$softiocList) {
          #print "host=$host ioc=$softioc,'p',switch=$switch,blade=$blade,port=$port \n";   
          push(@$network_hits_ref,[$softioc,'p',$switch,$blade,$port]);   
          email_log('debug',"Network data: $softioc-p NEW:$switch:$bladePort");   
            next;
        }
        next;
      } else {
        email_log('verbose',"Skipping: $line");   
        next;
      }

    }
    $host = substr($iocEtype,0,length($iocEtype)-2);
    $Etype = substr($iocEtype,length($iocEtype)-1,1);

    if ( $host =~ m/^hub/  || !defined $Etype  )  {
      email_log('verbose',"Skipping: $line");   
      next;
    }

    $ioc = $host;

    if ( $Etype !~ m/s/ &&  $Etype !~ m/p/ )  {
        email_log('verbose',"Skipping: $line");   
        next;
    }

    # May have to add ioc prefix
    if ( $ioc !~ m/^ioc/ ) {
      email_log('warn',"Prefix `ioc` was added to $ioc.");   
      $ioc =~ s/^/ioc/ 
    }

    push(@$network_hits_ref,[$ioc,$Etype,$switch,$blade,$port]);   
    email_log('debug',"Network data: $ioc-$Etype NEW:$switch:$bladePort");   
  }
  close(INFILE);

  return 0;
}


#=============================================================================
# Create a hash of hostnames containing a list of softiocnames on the host.
  
# Private
sub createHostHash {
  my $filename;
  my $hostname;
  my $softioc;

  push (@filenameList, glob("$bootparams/sioc*"));
  foreach $filename (@filenameList) {
    $hostname = getHostname($filename);
    next if $hostname eq '';
    my $softioc=$filename;
    $softioc =~ s/^.*\///;
    push(@$hostname,$softioc);
    $hostHash{$hostname}= \@$hostname if !exists $hostHash{$hostname};
  }
}


#=============================================================================
# Get host name for a soft ioc name from bootparams file.
# Private
sub getHostname {
  my ($iocname) = @_;
  local *IN;
  my $line;
  my $item;
  my ($name,$value);
  $iocname =~ s/^.*\///;
  return if $iocname !~ m/sioc/;
  if (!open(IN, "$bootparams/$iocname") ) {
    print "   Can't open bootparams file $iocname: $!\n";
    return "";
  }
  while ($line = <IN>) {
    chomp($line);
    (@chunks) = split (/ |,|\(|\)/,$line);
    foreach $item (@chunks) {
      ($name,$value) = split (/=/,$item);
       $value =~ s/\..*//;
       if ( $name eq "hn" ) {
        return $value;
       }
    }
  }
  return "";
}

#=============================================================================
# Public
sub send_email {
  my $body = $email_body."\n";
  my $from_address = "updateIocNetworkData";
#  my $to_address = "jba";
  my $to_address = "nda,jba";
  my $subject = "ioc network data updates";

  $mailer = Mail::Mailer->new();
  $mailer->open({ From	=> $from_address,
                To	=> $to_address,
                Subject	=> $subject,
              }) or die "Can't open: $! \n";

  print $mailer $body;
  $mailer->close();
}

#=============================================================================
# Public
sub email_log {
  if ( $_[0] ne 'debug' && $_[0] ne 'verbose' ) {
    $email_body = $email_body.uc($_[0]).": ".$_[1]."\n";
  }
  LogUtil::log($_[0],$_[1]);
}

1;

